/*                                                                                 
 *  confmgr3.c: Configuration utilities
 *
 *  Copyright (C) 2010- John Coppens (john@jcoppens.com)        
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 3 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *  
 *  You should have received a copy of the GNU Library General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <gconf/gconf-client.h>

#include "confmgr3.h"

void
confd_get_widgets_from_builder(GtkBuilder *builder, conf_definition *tbl)
{
  int i;
    
  for (i = 0; tbl[i].wdg_name != NULL; i++) {
    tbl[i].wdg = GTK_WIDGET(gtk_builder_get_object(builder, tbl[i].wdg_name));
    if (!tbl[i].wdg)
      printf("Widget not found: %s\n", tbl[i].wdg_name);
  }
}

void
confd_set_glade2_widgets(conf_definition *tbl, GtkWidget *ref,
		lookup_function lookup_widget)
{
  int i;
    
  for (i = 0; tbl[i].wdg_name != NULL; i++) {
    tbl[i].wdg = lookup_widget(ref, tbl[i].wdg_name);
  }
}

void
confd_set_widgets_to_values(conf_definition *tbl)
{
  char *p;
  int i;

  for (i = 0; tbl[i].wdg_name != NULL; i++) {
    if (!tbl[i].wdg) continue;			// No widget to set
    if (!tbl[i].var) continue;			// No corresponding variable
    switch (tbl[i].type) {
      case CDEF_NOVALUE:
	break;
      case CDEF_ENTRY2STR:
        if ((char **)tbl[i].var)
	  gtk_entry_set_text(GTK_ENTRY(tbl[i].wdg), *(char **)tbl[i].var);
 	else
 	  gtk_entry_set_text(GTK_ENTRY(tbl[i].wdg), "");
        break;
      case CDEF_ENTRY2INT:
	gtk_entry_set_text(GTK_ENTRY(tbl[i].wdg),
		p = g_strdup_printf("%d", *(int *)tbl[i].var));
	g_free(p);
        break;
      case CDEF_ENTRY2DBL:
	gtk_entry_set_text(GTK_ENTRY(tbl[i].wdg),
		p = g_strdup_printf("%.*f", tbl[i].aux, *(double *) tbl[i].var));
	g_free(p);
        break;
      case CDEF_CBENTRY2STR:
      case CDEF_CBBOX2STR:
	set_combo_box_store_to_string(GTK_COMBO_BOX(tbl[i].wdg), *(char **)tbl[i].var);
        break;
      case CDEF_CBBOX2INT:
        gtk_combo_box_set_active(GTK_COMBO_BOX(tbl[i].wdg), *(int *)tbl[i].var);
        break;
      case CDEF_SPBTN2INT:
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(tbl[i].wdg), *(int *)tbl[i].var);
        break;
      case CDEF_RBTN2CHAR:
      case CDEF_CBTN2CHAR:
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tbl[i].wdg), *(char *)tbl[i].var);
        break;
      case CDEF_CBTN2BIT:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tbl[i].wdg),
		*(int *)tbl[i].var & (1 << tbl[i].aux));
	break;
      case CDEF_LABEL2STR:
	gtk_label_set_label(GTK_LABEL(tbl[i].wdg), *(char **)tbl[i].var);
	break;
      case CDEF_LABEL2DBL:
	gtk_label_set_label(GTK_LABEL(tbl[i].wdg), 
		p = g_strdup_printf("%.*f", tbl[i].aux, *(double *) tbl[i].var));
	g_free(p);
        break;
      case CDEF_LABEL2INT:
	gtk_label_set_label(GTK_LABEL(tbl[i].wdg),
		p = g_strdup_printf("%d", *(int *)tbl[i].var));
	g_free(p);
	break;
      case CDEF_FONTBTN2STR:
	gtk_font_button_set_font_name(GTK_FONT_BUTTON(tbl[i].wdg), *(char **)tbl[i].var);
        break;
      case CDEF_DATE2INT:
	gtk_label_set_label(GTK_LABEL(tbl[i].wdg),
		p = g_strdup_printf("%d", *(int *)tbl[i].var));
	g_free(p);
	break;

    }
  }
}

void
confd_get_values_from_widgets(conf_definition *tbl)
{
  int i;

  for (i = 0; tbl[i].wdg_name != NULL; i++) {
    if (!tbl[i].wdg) continue;			// If no widget, continue
    if (!tbl[i].var) continue;			// No corresponding variable
    switch (tbl[i].type) {
      case CDEF_NOVALUE:
	break;
      case CDEF_ENTRY2STR:
        if (*(char **)tbl[i].var) g_free(*(char **)tbl[i].var);
        *(char **)tbl[i].var = g_strdup(gtk_entry_get_text(GTK_ENTRY(tbl[i].wdg)));
        break;
      case CDEF_ENTRY2INT:
	*(int *)tbl[i].var = atoi(gtk_entry_get_text(GTK_ENTRY(tbl[i].wdg)));
        break;
      case CDEF_ENTRY2DBL:
        *(double *) tbl[i].var = atof(gtk_entry_get_text(GTK_ENTRY(tbl[i].wdg)));
        break;
      case CDEF_CBENTRY2STR:
      case CDEF_CBBOX2STR:
        if (*(char* *)tbl[i].var) g_free(*(char **)tbl[i].var);
        *(char **)tbl[i].var = gtk_combo_box_get_active_text(GTK_COMBO_BOX(tbl[i].wdg));
        break;
      case CDEF_CBBOX2INT:
        *(int *)tbl[i].var = gtk_combo_box_get_active(GTK_COMBO_BOX(tbl[i].wdg));
        break;
      case CDEF_SPBTN2INT:
	*(int *)tbl[i].var = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tbl[i].wdg));
        break;
      case CDEF_RBTN2CHAR:
      case CDEF_CBTN2CHAR:
	*(char *)tbl[i].var = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tbl[i].wdg));
        break;
      case CDEF_CBTN2BIT:
        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tbl[i].wdg)))
          *(int *)tbl[i].var |= 1 << tbl[i].aux;
        else
          *(int *)tbl[i].var &= ~(1 << tbl[i].aux);
	break;
      case CDEF_LABEL2STR:
        if (*(char **)tbl[i].var) g_free(*(char **)tbl[i].var);
	*(char **)tbl[i].var = g_strdup(gtk_label_get_label(GTK_LABEL(tbl[i].wdg)));
	break;
      case CDEF_LABEL2DBL:
	*(double *) tbl[i].var = atof(gtk_label_get_label(GTK_LABEL(tbl[i].wdg)));
        break;
      case CDEF_LABEL2INT:
        *(int *)tbl[i].var = atoi(gtk_label_get_label(GTK_LABEL(tbl[i].wdg)));
	break;
      case CDEF_FONTBTN2STR:
	*(char **)tbl[i].var =
		g_strdup(gtk_font_button_get_font_name(GTK_FONT_BUTTON(tbl[i].wdg)));
        break;
      case CDEF_DATE2INT:
	*(int *)tbl[i].var = atoi(gtk_entry_get_text(GTK_ENTRY(tbl[i].wdg)));
	break;
    }
  }
}

void
confd_clear_widgets_to_defaults(conf_definition *tbl)
{
  char *p;
  int i;

  for (i = 0; tbl[i].wdg_name != NULL; i++) {
    if (!tbl[i].wdg) continue;			// If no widget, continue
    if (!tbl[i].var) continue;			// No corresponding variable
    switch (tbl[i].type) {
      case CDEF_NOVALUE:
	break;
      case CDEF_ENTRY2STR:
      case CDEF_ENTRY2INT:
      case CDEF_ENTRY2DBL:
	if (tbl[i].def)
	  gtk_entry_set_text(GTK_ENTRY(tbl[i].wdg), (char *)tbl[i].def);
	else
	  gtk_entry_set_text(GTK_ENTRY(tbl[i].wdg), "");
	break;
      case CDEF_SPBTN2INT:
	if (tbl[i].def)
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(tbl[i].wdg),
		atoi((char *)tbl[i].def));
	else
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(tbl[i].wdg), 0);
	break;
      case CDEF_LABEL2INT:
	if (tbl[i].def)
          gtk_label_set_label(GTK_LABEL(tbl[i].wdg), tbl[i].def);
	else
	  gtk_label_set_label(GTK_LABEL(tbl[i].wdg), "0");
	break;
      case CDEF_FONTBTN2STR:
	if (tbl[i].def) 
	  gtk_font_button_set_font_name(GTK_FONT_BUTTON(tbl[i].wdg), (char *)tbl[i].def);
	else
	  gtk_font_button_set_font_name(GTK_FONT_BUTTON(tbl[i].wdg), "");
        break;
    }
  }
}

void
set_combo_box_store_to_string(GtkComboBox *cbbox, char *str)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  gboolean ok;
  char *p;

  if (!str) {                   // Could happend if gconf doesn't find the valu
    gtk_combo_box_set_active(cbbox, 0);
    return;
  }
  model = gtk_combo_box_get_model(cbbox);
  ok = gtk_tree_model_get_iter_first(model, &iter);
  while (ok) {
    gtk_tree_model_get(model, &iter, 0, &p, -1);
    if (strcmp(p, str) == 0) {
      gtk_combo_box_set_active_iter(cbbox, &iter);
      break;
    }       
    ok = gtk_tree_model_iter_next(model, &iter);
  }
  if (!ok) 
    gtk_combo_box_set_active(cbbox, 0);
}

void
load_combo_box_with_strings(GtkComboBox *cbbox, char **str)
{
  GtkListStore *store;
  GtkTreeIter iter;
  int i;

  store = GTK_LIST_STORE(gtk_combo_box_get_model(cbbox));
  gtk_list_store_clear(store);

  for (i = 0; str[i]; i++) {
    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, str[i], -1);
  }

  gtk_combo_box_set_active(cbbox, 0);
}


void
load_by_table(conf_definition *tbl, char *path)
{                                                
  int i, v;
  char *tmp, *fullpath;
  GError *err = NULL;
  GConfClient *client;
  GConfValue *value;

  client = gconf_client_get_default();
  
  for (i = 0; tbl[i].wdg_name != NULL; i++) {
    fullpath = g_strdup_printf("%s/%s", path, tbl[i].keyname);
    value = gconf_client_get_without_default(client, fullpath, &err);

    switch (tbl[i].type) {
      if (!tbl[i].var) continue;

      case CDEF_NOVALUE:                                                  
        break;

      case CDEF_CBENTRY2STR:
      case CDEF_CBBOX2STR:
      case CDEF_LABEL2STR:
      case CDEF_FONTBTN2STR:
      case CDEF_ENTRY2STR:
        if (value)
          *(char **)tbl[i].var = g_strdup_printf("%s", gconf_value_get_string(value));
        else
          *(char **)tbl[i].var = g_strdup_printf("%s", (char *)tbl[i].def);
        break;

      case CDEF_LABEL2DBL:
      case CDEF_ENTRY2DBL:
        if (value)
          *(double *)tbl[i].var = gconf_value_get_float(value);
        else
          *(double *)tbl[i].var = atof(tbl[i].def);
        break;

      case CDEF_CBBOX2INT:
      case CDEF_SPBTN2INT:
      case CDEF_LABEL2INT:
      case CDEF_ENTRY2INT:
        if (value)
          *(int *)tbl[i].var = gconf_value_get_int(value);
        else 
          sscanf(tbl[i].def, "%i", (int *)tbl[i].var);
        break;                                     

      case CDEF_RBTN2CHAR:                         
      case CDEF_CBTN2CHAR:
        if (value) 
          *(char *)tbl[i].var = gconf_value_get_bool(value); 
        else {
          sscanf(tbl[i].def, "%i", &v);
          *(char *)tbl[i].var = (char) v;
        }
        break;

      default:
        printf("Confmgr error: Unknown type in table\n");
    }
    if (err)
      printf("Error in gconf: %s\n(Path: %s\n)", err->message, fullpath);
    g_free(fullpath);
    if (value)
      gconf_value_free(value);
  }
  g_object_unref(G_OBJECT(client));                
}


int
save_by_table(conf_definition *tbl, char *path)   
{
  int i;
  char *fullpath;
  GError *err = NULL;
  GConfClient *client;

  client = gconf_client_get_default();
 
  for (i = 0; tbl[i].wdg_name != NULL; i++) {
    if (!tbl[i].var) continue;
    fullpath = g_strdup_printf("%s/%s", path, tbl[i].keyname);
    switch (tbl[i].type) {
      case CDEF_NOVALUE:
        break;
      case CDEF_CBENTRY2STR:
      case CDEF_CBBOX2STR:
      case CDEF_LABEL2STR:
      case CDEF_FONTBTN2STR:
      case CDEF_ENTRY2STR:
        gconf_client_set_string(client, fullpath, *(char **)tbl[i].var, &err);
        break;
      case CDEF_LABEL2DBL:
      case CDEF_ENTRY2DBL:
        gconf_client_set_float(client, fullpath, *(double *)tbl[i].var, &err);
        break;
      case CDEF_CBBOX2INT:
      case CDEF_SPBTN2INT:  
      case CDEF_LABEL2INT:  
      case CDEF_ENTRY2INT:
        gconf_client_set_int(client, fullpath, *(int *)tbl[i].var, &err);
        break;
      case CDEF_RBTN2CHAR:
      case CDEF_CBTN2CHAR:
        gconf_client_set_bool(client, fullpath, *(char *)tbl[i].var, &err);
        break;
      default:
        printf("Confmgr error: Unknown type in table\n");
    }
    if (err)
      printf("Error in gconf: %s\n(Path: %s\n)", err->message, path);
    g_free(fullpath);
  }
  gconf_client_suggest_sync(client, &err);
  if (err) {
    printf("Confmgr error: %s", err->message);
  }
  g_object_unref(G_OBJECT(client));
  return (err == NULL);
}
